@@ -39,14 +39,36 @@ class Data:
         else:
             self.data[id] = item
 
+    def addFromMySQL(self, row):
+
+        # if row['milestone'] == 'TRUE':
+        #     row['milestone'] = True
+        # else:
+        #     row['milestone'] = False
+
+        id = row['id']
+        parent = row['parent']
+        row['start'] = row['start'].strftime('%Y-%m-%d')
+        row['finish'] = row['finish'].strftime('%Y-%m-%d')
+        if not row['resource']:
+            row['resource'] = ''
+        # print(type(parent))
+        if parent and parent > 0:
+            if not 'subitem' in self.data[parent]:
+                self.data[parent]['subitem'] = {}
+            self.data[parent]['subitem'][id] = row
+
+        else:
+            self.data[id] = row
+
     def addDict(self, item):
         pass
 
 
 class Gantt(Canvas):
     draw = None
-    canvasWidth = 1980
-    canvasHeight = 1080
+    canvasWidth = 0
+    canvasHeight = 0
     columeWidth = 30
     splitLine = 1
     canvasTop = 0
@@ -72,6 +94,11 @@ class Gantt(Canvas):
     data = {}
 
     def __init__(self) -> None:
+        super().__init__()
+        self.canvasWidth = self.width
+        self.canvasHeight = self.height
+        self.workweeks = 5
+        self.firstsd = None
         pass
 
     def title(self, text):
@@ -89,13 +116,20 @@ class Gantt(Canvas):
         top = 10
         self.draw.append(draw.Text("https://www.netkiller.cn - design by netkiller",
                                    15, self.canvasWidth - 280, top + 30, text_anchor='start', fill='grey'))
-        self.draw.append(draw.Rectangle(0, 0, self.canvasWidth - 1,
-                                        self.canvasHeight-1, fill='none', stroke='black'))
+        self.draw.append(draw.Rectangle(0, 0, self.canvasWidth,
+                                        self.canvasHeight, fill='none', stroke='black'))
         # fill='#eeeeee'
+        self.draw.append(draw.Image(
+            8, 8, 100, 34.99, 'by-nc-sa.png', embed=True))
 
     def hideTable(self):
         self.isTable = True
 
+    def setWorkweeks(self, workweeks=5, firstsd=False):
+        # 'five-day','six-day'
+        self.workweeks = workweeks
+        self.firstsd = firstsd
+
     def __table(self, top):
         group = draw.Group(id='table')
         group.append_title('表格')
@@ -121,6 +155,15 @@ class Gantt(Canvas):
 
         return group
 
+    def weekNumberOfMonth(self, currentDate):
+
+        # firstDay = currentDate.replace(            day=1, hour=0, minute=0, second=0, microsecond=0)
+        firstDay = currentDate.replace(day=1)
+        number = int(currentDate.strftime('%W')) - \
+            int(firstDay.strftime('%W')) + 1
+        # print(currentDate, number)
+        return number
+
     def __weekdays(self, top, begin, end):
         offsetX = 1
         column = 0
@@ -143,25 +186,35 @@ class Gantt(Canvas):
         endDay = end.day
         # print(beginDay, endDay)
 
-        weekNumber = datetime.strptime(
+        weekNumberOfYear = datetime.strptime(
             str(begin.year)+'-'+str(begin.month)+'-01', '%Y-%m-%d').strftime('%W')
-        # weekNumber = begin.strftime('%W')
-        # weekNumber = datetime.date(datetime.now().year,month,1).strftime('%W')
+        # weekNumberOfYear = begin.strftime('%W')
+        # weekNumberOfYear = datetime.date(datetime.now().year,month,1).strftime('%W')
         weekGroups = {}
-        weekGroups[weekNumber] = draw.Group(id='week'+str(weekNumber))
+        weekGroups[weekNumberOfYear] = draw.Group(
+            id='week'+str(weekNumberOfYear))
 
         for day in range(beginDay, endDay+1):
-            # print(day)
+
+            # numberOfWeek = self.weekNumberOfMonth(datetime.strptime(str(begin.year)+'-'+str(begin.month)+'-'+str(day), '%Y-%m-%d').date())
+            # print()
+            # x;qi
             weekday = calendar.weekday(begin.year, begin.month, day)
 
-            currentWeekNumber = datetime.strptime(
+            currentweekNumberOfYear = datetime.strptime(
                 str(begin.year) + '-'+str(begin.month)+'-' + str(day), '%Y-%m-%d').strftime('%W')
-            # print(weekNumber, currentWeekNumber)
-            if currentWeekNumber != weekNumber:
-                weekNumber = currentWeekNumber
-                weekGroups[weekNumber] = draw.Group(id='week'+str(weekNumber))
-
-            if weekday >= 5:
+            # print(weekNumberOfYear, currentweekNumberOfYear)
+            if currentweekNumberOfYear != weekNumberOfYear:
+                weekNumberOfYear = currentweekNumberOfYear
+                weekGroups[weekNumberOfYear] = draw.Group(
+                    id='week'+str(weekNumberOfYear))
+            # if self.firstsd == True:
+            #     self.workweeks = 6
+            #     self.firstsd = False
+            # else:
+            #     self.workweeks = 5
+            #     self.firstsd = True
+            if weekday >= self.workweeks:
                 color = '#dddddd'
             else:
                 color = '#cccccc'
@@ -169,20 +222,19 @@ class Gantt(Canvas):
             x = self.weekdayPosition + self.columeWidth * (column) + offsetX
             self.dayPosition[date(year=int(begin.year), month=int(
                 begin.month), day=int(day)).strftime('%Y-%m-%d')] = x
-            if weekday == 6:
-                weekGroups[weekNumber].append(draw.Line(x + self.columeWidth, top + self.itemHeight,
-                                                        x + self.columeWidth, self.canvasHeight, stroke='black'))
+
             if day == beginDay:
-                weekGroups[weekNumber].append(draw.Text(begin.strftime(
+                weekGroups[weekNumberOfYear].append(draw.Text(begin.strftime(
                     '%Y年%m月'), 20, x + 4, top + self.itemHeight - 10, fill='#555555'))
+            # 右侧封闭
             if day == endDay:
-                weekGroups[weekNumber].append(draw.Line(x + self.columeWidth, top,
-                                                        x + self.columeWidth, self.canvasHeight, stroke='black'))
+                weekGroups[weekNumberOfYear].append(draw.Line(x + self.columeWidth, top,
+                                                              x + self.columeWidth, self.canvasHeight, stroke='black'))
 
             # dayName = ["星期一","星期二","星期三","星期四","星期五","星期六","星期日"]
             dayName = ["一", "二", "三", "四", "五", "六", "日"]
 
-            weekGroups[weekNumber].append(
+            weekGroups[weekNumberOfYear].append(
                 draw.Text(dayName[weekday], 20, x + 4, top + self.columeWidth*2-10, fill='#555555'))
             if day < 10:
                 numberOffsetX = 10
@@ -194,10 +246,13 @@ class Gantt(Canvas):
             r = draw.Rectangle(x, top+self.itemHeight*2, self.columeWidth,
                                self.canvasHeight - (top+self.itemHeight*2), fill=color)
             r.append_title(str(day))
-            weekGroups[weekNumber].append(r)
-
+            weekGroups[weekNumberOfYear].append(r)
+            # 周分割线
+            if weekday == 6:
+                weekGroups[weekNumberOfYear].append(draw.Line(x + self.columeWidth, top + self.itemHeight,
+                                                              x + self.columeWidth, self.canvasHeight, stroke='black'))
             # 日期
-            weekGroups[weekNumber].append(
+            weekGroups[weekNumberOfYear].append(
                 draw.Text(str(day), 20, x + numberOffsetX, top + self.columeWidth * 3 - 10, fill='#555555'))
 
             # if column:
@@ -456,8 +511,6 @@ class Gantt(Canvas):
         self.startPosition = 400
         left = self.startPosition
 
-        # self.title(title)
-
         for key, value in self.data.items():
             self.fontSize = self.getTextSize(key)
 
@@ -473,17 +526,16 @@ class Gantt(Canvas):
                 self.endDate = finish
 
         lineNumber = len(self.data)
-        print(lineNumber)
+
         days = self.endDate - self.beginDate
         self.canvasWidth = self.startPosition + self.columeWidth * \
             days.days + days.days + self.columeWidth + 2
         self.canvasHeight = self.canvasTop + self.itemHeight * \
             5 + self.itemHeight * lineNumber + lineNumber + 15
-        print(self.canvasTop, self.canvasHeight)
+        # print(self.canvasTop, self.canvasHeight)
 
         self.draw = draw.Drawing(self.canvasWidth, self.canvasHeight)
-        self.draw.append(draw.Rectangle(0, 0, self.canvasWidth - 1,
-                                        self.canvasHeight-1, fill='#eeeeee', stroke='black'))
+
         self.title(title)
 
         top = self.itemHeight * 4 - 10
@@ -508,8 +560,7 @@ class Gantt(Canvas):
                      self.nameTextSize + 300, self.canvasHeight, stroke='grey'))
         table.append(draw.Text('工时', 20, self.nameTextSize +
                                300, top + 20, fill='#555555'))
-        table.append(draw.Line(self.nameTextSize + 400, top,
-                               self.nameTextSize + 400, self.canvasHeight, stroke='grey'))
+        # table.append(draw.Line(self.nameTextSize + 400, top,                               self.nameTextSize + 400, self.canvasHeight, stroke='grey'))
 
         chart.append(table)
 
@@ -522,14 +573,15 @@ class Gantt(Canvas):
 
         # for key, value in self.__weekday(top).items():
         #     background.append(value)
-
-        chart.append(draw.Line(1, self.canvasTop + 26, self.canvasWidth,
-                               self.canvasTop + 26, stroke='grey'))
-
+        # 月线
+        chart.append(draw.Line(self.startPosition, self.canvasTop + self.itemHeight, self.canvasWidth,
+                               self.canvasTop + self.itemHeight, stroke='grey'))
+        # 周线
         chart.append(draw.Line(1, self.canvasTop + self.itemHeight * 2, self.canvasWidth,
                                self.canvasTop + self.itemHeight * 2, stroke='grey'))
-        chart.append(draw.Line(1, self.canvasTop + self.itemHeight * 3, self.canvasWidth,
-                               self.canvasTop + self.itemHeight * 3, stroke='black'))
+
+        # chart.append(draw.Line(1, self.canvasTop + self.itemHeight * 3, self.canvasWidth,
+        #                        self.canvasTop + self.itemHeight * 3, stroke='black'))
         # 竖线
         chart.append(
             draw.Line(left, self.canvasTop, left, self.canvasHeight, stroke='grey'))
@@ -570,11 +622,14 @@ class Gantt(Canvas):
             chart.append(r)
 
             chart.append(draw.Line(1, top + self.itemHeight, self.canvasWidth,
-                                   top + self.itemHeight, stroke='grey'))
+                         top + self.itemHeight, stroke='grey'))
 
             self.itemLine += 1
 
         self.draw.append(chart)
+        # self.draw.append(draw.Rectangle(1, 1, self.canvasWidth,
+        #                                 self.canvasHeight, fill='none', stroke='black'))
+        self.legend()
 
     def getTextSize(self, text, size=21):
 
@@ -587,8 +642,9 @@ class Gantt(Canvas):
         # size = cv2.getTextSize(text, fontFace, fontScale, thickness)
         # width, height = size[0]
 
-        font = 'AdobeFangsongStd-Regular.otf'
+        # font = 'AdobeFangsongStd-Regular.otf'
         # font = r'Apple Symbols.ttf'
+        font = r'/System/Library/Fonts/Supplemental/Songti.ttc'
         font = ImageFont.truetype(font=font, size=size, encoding="utf-8")
         # font = ImageFont.truetype("symbol.ttf", 16, encoding="symb")
         width, height = font.getsize(text)
@@ -610,7 +666,7 @@ class Gantt(Canvas):
             if self.nameTextSize < length:
                 self.nameTextSize = length
 
-        if 'resource' in item:
+        if 'resource' in item and item['resource']:
             length = self.getTextSize(item['resource'])
             if self.resourceTextSize < length:
                 self.resourceTextSize = length
@@ -645,7 +701,8 @@ class Gantt(Canvas):
         # print(self.maxDate, end)
         # print(self.beginDate, self.endDate)
 
-        # self.nameTextSize += self.textIndentSize
+        # 行首加5像素美化
+        self.nameTextSize += 10
 
         if not self.isTable:
             self.startPosition = self.nameTextSize + self.resourceTextSize + 250
